@@ -78,7 +78,7 @@ class AbstractAuth(ABC):  # pylint: disable=too-few-public-methods
         if not (url.startswith("http://") or url.startswith("https://")):
             url = f"{self._host}/{url}"
         _LOGGER.debug("request[%s]=%s %s", method, url, kwargs.get("params"))
-        if method == "post" and "json" in kwargs:
+        if method != "get" and "json" in kwargs:
             _LOGGER.debug("request[post json]=%s", kwargs["json"])
         return await self._websession.request(method, url, **kwargs, headers=headers)
 
