@@ -10,11 +10,16 @@ from __future__ import annotations
 import datetime
 import logging
 import zoneinfo
+from collections.abc import Iterable
 from enum import Enum
 from typing import Any, Optional, Union
 
 from dateutil import rrule
+from ical.component import ComponentModel
+from ical.iter import RecurIterable
+from ical.parsing.component import parse_content
 from ical.timespan import Timespan
+from ical.types.data_types import DATA_TYPE
 from ical.types.recur import Recur
 from pydantic import BaseModel, Field, root_validator
 
@@ -117,12 +122,23 @@ class DateOrDatetime(BaseModel):
 
     @property
     def value(self) -> Union[datetime.date, datetime.datetime]:
-        """Return either a datetime or date representing the Datetime."""
+        """Return either a datetime or date representing the Datetime.
+
+        This only replace tzinfo/offset in the datetime when the API response did not specify one
+        separately, preferring the offset specified in the event response.
+        """
         if self.date is not None:
             return self.date
         if self.date_time is not None:
-            if self.date_time.tzinfo is None and self.timezone is not None:
-                return self.date_time.replace(tzinfo=zoneinfo.ZoneInfo(self.timezone))
+            if self.timezone is not None:
+                # Always use the timezone when there isn't one, otherwise only override when
+                # using to start recurring events. Otherwise, just use the simple offset
+                # specified in the event.
+                if self.date_time.tzinfo is None:
+                    return self.date_time.replace(
+                        tzinfo=zoneinfo.ZoneInfo(self.timezone)
+                    )
+                return self.date_time.astimezone(tz=zoneinfo.ZoneInfo(self.timezone))
             return self.date_time
         raise ValueError("Datetime has invalid state with no date or date_time")
 
@@ -310,6 +326,84 @@ class SyntheticEventId:
         return self._dtstart
 
 
+class Recurrence(ComponentModel):
+    """A pydantic model that captures the objects in a Google Calendar recurrence."""
+
+    rrule: list[Recur] = []
+    """A recurrence rule specification.
+
+    Defines a rule for specifying a repeated event. The recurrence set is the complete
+    set of recurrence instances for a calendar component (based on rrule, rdate, exdate).
+
+    The recurrence set is generated by gathering the rrule and rdate properties then
+    excluding any times specified by exdate. The recurrence is generated with the dtstart
+    property defining the first instance of the recurrence set.
+
+    Typically a dtstart should be specified with a date local time and timezone to make
+    sure all instances have the same start time regardless of time zone changing.
+    """
+
+    rdate: list[Union[datetime.datetime, datetime.date]] = Field(default_factory=list)
+    """Defines the list of date/time values for recurring events.
+
+    Can appear along with the rrule property to define a set of repeating occurrences of the
+    event. The recurrence set is the complete set of recurrence instances for a calendar component
+    (based on rrule, rdate, exdate). The recurrence set is generated by gathering the rrule
+    and rdate properties then excluding any times specified by exdate.
+    """
+
+    exdate: list[Union[datetime.datetime, datetime.date]] = Field(default_factory=list)
+    """Defines the list of exceptions for recurring events.
+
+    The exception dates are used in computing the recurrence set. The recurrence set is
+    the complete set of recurrence instances for a calendar component (based on rrule, rdate,
+    exdate). The recurrence set is generated by gathering the rrule and rdate properties
+    then excluding any times specified by exdate.
+    """
+
+    @classmethod
+    def from_recurrence(cls, recurrence: list[str]) -> "Recurrence":
+        """Parse a Recurrence object form calendar API list of recurrence rules."""
+        content = (
+            [
+                "BEGIN:RECURRENCE",
+            ]
+            + recurrence
+            + [
+                "END:RECURRENCE",
+            ]
+        )
+        component = parse_content("\n".join(content))
+        return cls.parse_obj(component[0].as_dict())
+
+    def as_rrule(
+        self, dtstart: datetime.date | datetime.datetime
+    ) -> Iterable[datetime.date | datetime.datetime]:
+        """Return the set of recurrences as a rrule that emits start times."""
+        ruleset = rrule.rruleset()
+        for rule in self.rrule:
+            ruleset.rrule(rule.as_rrule(dtstart))
+        for rdate in self.rdate:
+            ruleset.rdate(rdate)  # type: ignore[no-untyped-call]
+        for exdate in self.exdate:
+            if not isinstance(exdate, datetime.datetime):
+                # Convert to datetime matching dateutil's logic
+                exdate = datetime.datetime.fromordinal(exdate.toordinal())
+            ruleset.exdate(exdate)  # type: ignore[no-untyped-call]
+        return RecurIterable(lambda x: x, ruleset)
+
+    def as_recurrence(self) -> list[str]:
+        """Serialize the recurrence rule as an API string."""
+        return [prop.ics() for prop in self.__encode_component_root__().properties]
+
+    class Config:
+        """Configuration for IcsCalendarStream pydantic model."""
+
+        json_encoders = DATA_TYPE.encode_property_json
+        validate_assignment = True
+        allow_population_by_field_name = True
+
+
 class Event(BaseModel):
     """A single event on a calendar."""
 
@@ -371,6 +465,8 @@ class Event(BaseModel):
 
     See RFC5545 for more details."""
 
+    recur: Optional[Recurrence] = Field(default=None, exclude=True)
+
     recurring_event_id: Optional[str] = Field(alias="recurringEventId", default=None)
     """The id of the primary even to which this recurring event belongs."""
 
@@ -385,21 +481,11 @@ class Event(BaseModel):
         return self.end.value - self.start.value
 
     @property
-    def rrule(self) -> rrule.rrule | rrule.rruleset:
+    def rrule(self) -> Iterable[Union[datetime.date, datetime.datetime]]:
         """Return the recurrence rules as a set of rules."""
-        try:
-            return rrule.rrulestr("\n".join(self.recurrence), dtstart=self.start.value)
-        except ValueError as err:
-            raise ValueError(
-                f"Invalid recurrence rule: {self.json()}: {str(err)}"
-            ) from err
-
-    @property
-    def recur(self) -> Recur:
-        """Build a recurrence rule for the event."""
-        if len(self.recurrence) != 1:
-            raise ValueError(f"Unexpected recurrence value: {self.recurrence}")
-        return Recur.from_rrule(self.recurrence[0])
+        if len(self.recurrence) == 0 or not self.recur:
+            return []
+        return self.recur.as_rrule(self.start.value)
 
     @root_validator(pre=True)
     def _allow_cancelled_events(cls, values: dict[str, Any]) -> dict[str, Any]:
@@ -420,59 +506,47 @@ class Event(BaseModel):
                 values["visibility"] = "private"
         return values
 
+    @root_validator(pre=True)
+    def _validate_recur(cls, values: dict[str, Any]) -> dict[str, Any]:
+        """Remove rrule property parameters not supported by the dateutil.rrule library."""
+        if not values.get("recurrence"):
+            return values
+        values["recur"] = Recurrence.from_recurrence(values["recurrence"])
+        return values
+
     @root_validator
     def _validate_rrule(cls, values: dict[str, Any]) -> dict[str, Any]:
         """The API returns invalid RRULEs that need to be coerced to valid."""
         # Rules may need updating of start time has a timezone
-        if not (recurrence := values.get("recurrence")) or not (
-            dtstart := values.get("start")
-        ):
+        if not (recur := values.get("recur")) or not (dtstart := values.get("start")):
             return values
-        values["recurrence"] = [cls._adjust_rrule(rule, dtstart) for rule in recurrence]
+        for rule in recur.rrule:
+            cls._adjust_rrule(rule, dtstart)
         return values
 
     @classmethod
-    def _adjust_rrule(cls, rule: str, dtstart: DateOrDatetime) -> str:
+    def _adjust_rrule(cls, rule: Recur, dtstart: DateOrDatetime) -> Recur:
         """Apply fixes to the rrule."""
-        if not rule.startswith("RRULE:"):
+        if not rule.until:
             return rule
 
-        parts = {}
-        for part in rule[6:].split(";"):
-            if "=" not in part:
-                raise ValueError(
-                    f"Recurrence rule had unexpected format missing '=': {rule}"
-                )
-            key, value = part.split("=", 1)
-            key = key.upper()
-            parts[key.upper()] = value
-
-        if not (until := parts.get("UNTIL")):
-            return rule
-
-        until_parts = until.split("T")
-        if len(until_parts) > 2:
-            raise ValueError(f"Recurrence rule had invalid UNTIL: {rule}")
-
         if dtstart.date_time:
-            if dtstart.date_time.tzinfo and len(until_parts) == 1:
-                # UNTIL is a DATE but must be a DATE-TIME
-                parts["UNTIL"] = f"{until}T000000Z"
-            elif dtstart.date_time.tzinfo is None and until_parts[1].endswith("Z"):
+            if dtstart.date_time and not isinstance(rule.until, datetime.datetime):
+                # Convert a date to a datetime
+                rule.until = datetime.datetime.fromordinal(
+                    rule.until.toordinal()
+                ).replace(tzinfo=dtstart.date_time.tzinfo)
+            elif (
+                dtstart.date_time.tzinfo is None
+                and isinstance(rule.until, datetime.datetime)
+                and rule.until.tzinfo is not None
+            ):
                 # Date should be floating
-                parts["UNTIL"] = f"{until_parts[0]}T{until_parts[1][:-1]}"
+                rule.until = rule.until.replace(tzinfo=None)
         elif dtstart.date:
-            if len(until_parts) > 1:
+            if isinstance(rule.until, datetime.datetime):
                 # UNTIL is a DATE-TIME but must be a DATE
-                parts["UNTIL"] = until_parts[0]
-
-        rule = ";".join(f"{k}={v}" for k, v in parts.items())
-        try:
-            rrule.rrulestr(rule, dtstart=dtstart.value)
-        except ValueError as err:
-            raise ValueError(
-                f"Invalid recurrence rule {rule} for {dtstart}: {str(err)}"
-            ) from err
+                rule.until = rule.until.date()
         return rule
 
     @property
